@@ -93,13 +93,13 @@ similarity_method: str = 'pearson'
 #: :py:func:`metacells.utilities.computation.logistics`.
 logistics_location: float = 0.8
 
-#: The default scale for the logistics function. See
+#: The default slope for the logistics function. See
 #: :py:func:`metacells.pipeline.umap.compute_umap_by_features`,
 #: :py:func:`metacells.tools.similarity.compute_obs_obs_similarity`,
 #: :py:func:`metacells.tools.similarity.compute_var_var_similarity`.
 #: and
 #: :py:func:`metacells.utilities.computation.logistics`.
-logistics_scale: float = 0.5
+logistics_slope: float = 0.5
 
 #: The generic target number of observations (cells) in a pile, allowing us to
 #: directly compute groups (metacells) for it. See
@@ -171,6 +171,14 @@ max_merge_size_factor: float = 0.25
 #: :py:func:`metacells.tools.dissolve.dissolve_metacells`.
 min_metacell_cells: int = 12
 
+#: The maximal strength of a min-cut of a metacell that will cause it to be split. See
+#: :py:func:`metacells.tools.candidates.compute_candidate_metacells`.
+max_split_min_cut_strength: float = 0.1
+
+#: The minimal number of cells to keep as a seed when cutting a metacell. See
+#: :py:func:`metacells.tools.candidates.compute_candidate_metacells`.
+min_cut_seed_cells: int = 7
+
 #: The minimal size factor of a pile, above which we can split it. See
 #: :py:const:`min_split_size_factor`,
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
@@ -465,7 +473,7 @@ feature_downsample_max_cell_quantile: float = downsample_max_cell_quantile
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
-feature_min_gene_relative_variance: float = 0.1
+feature_min_gene_relative_variance: Optional[float] = 0.1
 
 #: The minimal number of downsampled UMIs of a gene to be considered a "feature". See
 #: :py:func:`metacells.tools.high.find_high_total_genes`,
@@ -474,7 +482,7 @@ feature_min_gene_relative_variance: float = 0.1
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
-feature_min_gene_total: int = 50
+feature_min_gene_total: Optional[int] = 50
 
 #: The minimal number of the top-3rd downsampled UMIs of a gene to be considered a "feature". See
 #: :py:func:`metacells.tools.high.find_high_topN_genes`,
@@ -483,7 +491,7 @@ feature_min_gene_total: int = 50
 #: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
-feature_min_gene_top3: int = 4
+feature_min_gene_top3: Optional[int] = 4
 
 #: Whether to compute cell-cell similarity using the log (base 2) of the data. See
 #: :py:func:`metacells.pipeline.direct.compute_direct_metacells`,
@@ -508,12 +516,33 @@ cells_similarity_value_normalization: float = 1/significant_value
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
 cells_similarity_method: str = similarity_method
 
+#: Whether to compute group-group similarity using the log (base 2) of the data. See
+#: :py:func:`metacells.pipeline.direct.compute_direct_metacells`,
+#: :py:func:`metacells.pipeline.divide_and_conquer.compute_divide_and_conquer_metacells`
+#: and
+#: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
+groups_similarity_log_data: bool = True
+
 #: The method to use to compute group-group similarity. See
 #: :py:const:`cells_similarity_method`
 #: and
 #: :py:func:`metacells.pipeline.divide_and_conquer.divide_and_conquer_pipeline`.
 groups_similarity_method: str = similarity_method
 
+#: Whether to compute group self-similarity using the log (base 2) of the data. See
+#: :py:func:`metacells.pipeline.consistency.compute_groups_self_consistency`.
+self_similarity_log_data: bool = True
+
+#: The normalization factor to use after computing the fractions of the data for
+#: computing group self similarity. See
+#: :py:func:`metacells.pipeline.consistency.compute_groups_self_consistency`.
+self_similarity_value_normalization: float = 1e-5
+
+#: The method to use to compute group self-consistency. See
+#: :py:func:`metacells.tools.similarity.compute_obs_obs_similarity`,
+#: :py:func:`metacells.pipeline.consistency.compute_groups_self_consistency`.
+self_similarity_method: str = 'logistics'
+
 #: The target K for building the K-Nearest-Neighbors graph. See
 #: :py:func:`metacells.tools.knn_graph.compute_obs_obs_knn_graph`,
 #: :py:func:`metacells.tools.knn_graph.compute_var_var_knn_graph`,
