@@ -31,9 +31,11 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
     feature_downsample_min_samples: int = pr.feature_downsample_min_samples,
     feature_downsample_min_cell_quantile: float = pr.feature_downsample_min_cell_quantile,
     feature_downsample_max_cell_quantile: float = pr.feature_downsample_max_cell_quantile,
-    feature_min_gene_total: int = pr.feature_min_gene_total,
-    feature_min_gene_top3: int = pr.feature_min_gene_top3,
-    feature_min_gene_relative_variance: float = pr.feature_min_gene_relative_variance,
+    feature_min_gene_total: Optional[int] = pr.feature_min_gene_total,
+    feature_min_gene_top3: Optional[int] = pr.feature_min_gene_top3,
+    feature_min_gene_relative_variance: Optional[float] = pr.feature_min_gene_relative_variance,
+    feature_gene_names: Optional[Collection[str]] = None,
+    feature_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     forbidden_gene_names: Optional[Collection[str]] = None,
     forbidden_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     cells_similarity_value_normalization: float = pr.cells_similarity_value_normalization,
@@ -57,6 +59,8 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
     candidates_min_split_size_factor: Optional[float] = pr.candidates_min_split_size_factor,
     candidates_max_merge_size_factor: Optional[float] = pr.candidates_max_merge_size_factor,
     candidates_min_metacell_cells: Optional[int] = pr.min_metacell_cells,
+    candidates_max_split_min_cut_strength: Optional[float] = pr.max_split_min_cut_strength,
+    candidates_min_cut_seed_cells: Optional[int] = pr.min_cut_seed_cells,
     must_complete_cover: bool = False,
     deviants_min_gene_fold_factor: float = pr.deviants_min_gene_fold_factor,
     deviants_max_gene_fraction: Optional[float] = pr.deviants_max_gene_fraction,
@@ -158,11 +162,11 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
        ``feature_downsample_max_cell_quantile`` (default: {feature_downsample_max_cell_quantile}),
        ``feature_min_gene_total`` (default: {feature_min_gene_total}), ``feature_min_gene_top3``
        (default: {feature_min_gene_top3}), ``feature_min_gene_relative_variance (default:
-       {feature_min_gene_relative_variance}), ``forbidden_gene_names`` (default:
-       {forbidden_gene_names}), ``forbidden_gene_patterns`` (default: {forbidden_gene_patterns})
-       and
-       ``random_seed`` (default: {random_seed})
-       to make this replicable.
+       {feature_min_gene_relative_variance}), ``feature_gene_names`` (default:
+       {feature_gene_names}), ``feature_gene_patterns`` (default: {feature_gene_patterns}),
+       ``forbidden_gene_names`` (default: {forbidden_gene_names}), ``forbidden_gene_patterns``
+       (default: {forbidden_gene_patterns}) and ``random_seed`` (default: {random_seed}) to make
+       this replicable.
 
     2. Compute the fractions of each variable in each cell, and add the
        ``cells_similarity_value_normalization`` (default: {cells_similarity_value_normalization}) to
@@ -176,6 +180,11 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
        similarity between each pair of cells, using the
        ``cells_similarity_method`` (default: {cells_similarity_method}).
 
+    5. Invoke :py:func:`metacells.pipeline.collect.compute_effective_cell_sizes` using
+       ``max_cell_size`` (default: {max_cell_size}), ``max_cell_size_factor`` (default:
+       {max_cell_size_factor}) and ``cell_sizes`` (default: {cell_sizes}) to get the effective cell
+       sizes to use.
+
     5. Invoke :py:func:`metacells.tools.knn_graph.compute_obs_obs_knn_graph` to compute a
        K-Nearest-Neighbors graph, using the
        ``knn_balanced_ranks_factor`` (default: {knn_balanced_ranks_factor}),
@@ -201,8 +210,7 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
        ``random_seed`` (default: {random_seed})
        to make this replicable. This tries to build metacells of the
        ``target_metacell_size`` (default: {target_metacell_size})
-       using the
-       ``cell_sizes`` (default: {cell_sizes}).
+       using the effective cell sizes.
 
     7. Unless ``must_complete_cover`` (default: {must_complete_cover}), invoke
        :py:func:`metacells.tools.deviants.find_deviant_cells` to remove deviants from the candidate
@@ -216,8 +224,7 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
        :py:func:`metacells.tools.dissolve.dissolve_metacells` to dissolve small unconvincing
        metacells, using the same
        ``target_metacell_size`` (default: {target_metacell_size}),
-       and
-       ``cell_sizes`` (default: {cell_sizes}),
+       and the effective cell sizes
        and the
        ``dissolve_min_robust_size_factor`` (default: {dissolve_min_robust_size_factor}),
        ``dissolve_min_convincing_size_factor`` (default: {dissolve_min_convincing_size_factor}),
@@ -233,6 +240,8 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
                              min_gene_relative_variance=feature_min_gene_relative_variance,
                              min_gene_total=feature_min_gene_total,
                              min_gene_top3=feature_min_gene_top3,
+                             forced_gene_names=feature_gene_names,
+                             forced_gene_patterns=feature_gene_patterns,
                              forbidden_gene_names=forbidden_gene_names,
                              forbidden_gene_patterns=forbidden_gene_patterns,
                              random_seed=random_seed)
@@ -317,6 +326,9 @@ def compute_direct_metacells(  # pylint: disable=too-many-statements,too-many-br
                                        min_split_size_factor=candidates_min_split_size_factor,
                                        max_merge_size_factor=candidates_max_merge_size_factor,
                                        min_metacell_cells=candidates_min_metacell_cells,
+                                       max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                       min_cut_seed_cells=candidates_min_cut_seed_cells,
+                                       must_complete_cover=must_complete_cover,
                                        random_seed=random_seed)
 
         ut.set_oo_data(adata, 'obs_similarity',
