@@ -489,14 +489,17 @@ def divide_and_conquer_pipeline(
     feature_downsample_min_samples: int = pr.feature_downsample_min_samples,
     feature_downsample_min_cell_quantile: float = pr.feature_downsample_min_cell_quantile,
     feature_downsample_max_cell_quantile: float = pr.feature_downsample_max_cell_quantile,
-    feature_min_gene_total: int = pr.feature_min_gene_total,
-    feature_min_gene_top3: int = pr.feature_min_gene_top3,
-    feature_min_gene_relative_variance: float = pr.feature_min_gene_relative_variance,
+    feature_min_gene_total: Optional[int] = pr.feature_min_gene_total,
+    feature_min_gene_top3: Optional[int] = pr.feature_min_gene_top3,
+    feature_min_gene_relative_variance: Optional[float] = pr.feature_min_gene_relative_variance,
+    feature_gene_names: Optional[Collection[str]] = None,
+    feature_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     forbidden_gene_names: Optional[Collection[str]] = None,
     forbidden_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     cells_similarity_value_normalization: float = pr.cells_similarity_value_normalization,
     cells_similarity_log_data: bool = pr.cells_similarity_log_data,
     cells_similarity_method: str = pr.cells_similarity_method,
+    groups_similarity_log_data: bool = pr.groups_similarity_log_data,
     groups_similarity_method: str = pr.groups_similarity_method,
     target_pile_size: int = pr.target_pile_size,
     max_cell_size: Optional[float] = pr.max_cell_size,
@@ -519,6 +522,8 @@ def divide_and_conquer_pipeline(
     candidates_min_split_size_factor: Optional[float] = pr.candidates_min_split_size_factor,
     candidates_max_merge_size_factor: Optional[float] = pr.candidates_max_merge_size_factor,
     candidates_min_metacell_cells: Optional[int] = pr.candidates_min_metacell_cells,
+    candidates_max_split_min_cut_strength: Optional[float] = pr.max_split_min_cut_strength,
+    candidates_min_cut_seed_cells: Optional[int] = pr.min_cut_seed_cells,
     must_complete_cover: bool = False,
     final_max_outliers_levels: Optional[int] = pr.final_max_outliers_levels,
     deviants_min_gene_fold_factor: float = pr.deviants_min_gene_fold_factor,
@@ -726,6 +731,8 @@ def divide_and_conquer_pipeline(
                                         feature_min_gene_total=feature_min_gene_total,
                                         feature_min_gene_top3=feature_min_gene_top3,
                                         feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                        feature_gene_names=feature_gene_names,
+                                        feature_gene_patterns=feature_gene_patterns,
                                         forbidden_gene_names=forbidden_gene_names,
                                         forbidden_gene_patterns=forbidden_gene_patterns,
                                         cells_similarity_value_normalization=cells_similarity_value_normalization,
@@ -748,6 +755,8 @@ def divide_and_conquer_pipeline(
                                         candidates_min_split_size_factor=candidates_min_split_size_factor,
                                         candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                         candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                        candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                        candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                         must_complete_cover=False,
                                         deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
                                         deviants_max_gene_fraction=deviants_max_gene_fraction,
@@ -791,14 +800,17 @@ def divide_and_conquer_pipeline(
                                                  feature_downsample_min_samples=feature_downsample_min_samples,
                                                  feature_downsample_min_cell_quantile=feature_downsample_min_cell_quantile,
                                                  feature_downsample_max_cell_quantile=feature_downsample_max_cell_quantile,
-                                                 feature_min_gene_relative_variance=feature_min_gene_relative_variance,
                                                  feature_min_gene_total=feature_min_gene_total,
                                                  feature_min_gene_top3=feature_min_gene_top3,
+                                                 feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                                 feature_gene_names=feature_gene_names,
+                                                 feature_gene_patterns=feature_gene_patterns,
                                                  forbidden_gene_names=forbidden_gene_names,
                                                  forbidden_gene_patterns=forbidden_gene_patterns,
                                                  cells_similarity_value_normalization=cells_similarity_value_normalization,
                                                  cells_similarity_log_data=cells_similarity_log_data,
                                                  cells_similarity_method=cells_similarity_method,
+                                                 groups_similarity_log_data=groups_similarity_log_data,
                                                  groups_similarity_method=groups_similarity_method,
                                                  target_pile_size=target_pile_size,
                                                  pile_min_split_size_factor=pile_min_split_size_factor,
@@ -821,6 +833,8 @@ def divide_and_conquer_pipeline(
                                                  candidates_min_split_size_factor=candidates_min_split_size_factor,
                                                  candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                                  candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                                 candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                                 candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                                  must_complete_cover=must_complete_cover,
                                                  final_max_outliers_levels=final_max_outliers_levels,
                                                  deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
@@ -855,14 +869,17 @@ def compute_divide_and_conquer_metacells(
     feature_downsample_min_samples: int = pr.feature_downsample_min_samples,
     feature_downsample_min_cell_quantile: float = pr.feature_downsample_min_cell_quantile,
     feature_downsample_max_cell_quantile: float = pr.feature_downsample_max_cell_quantile,
-    feature_min_gene_total: int = pr.feature_min_gene_total,
-    feature_min_gene_top3: int = pr.feature_min_gene_top3,
-    feature_min_gene_relative_variance: float = pr.feature_min_gene_relative_variance,
+    feature_min_gene_total: Optional[int] = pr.feature_min_gene_total,
+    feature_min_gene_top3: Optional[int] = pr.feature_min_gene_top3,
+    feature_min_gene_relative_variance: Optional[float] = pr.feature_min_gene_relative_variance,
+    feature_gene_names: Optional[Collection[str]] = None,
+    feature_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     forbidden_gene_names: Optional[Collection[str]] = None,
     forbidden_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     cells_similarity_value_normalization: float = pr.cells_similarity_value_normalization,
     cells_similarity_log_data: bool = pr.cells_similarity_log_data,
     cells_similarity_method: str = pr.cells_similarity_method,
+    groups_similarity_log_data: bool = pr.groups_similarity_log_data,
     groups_similarity_method: str = pr.groups_similarity_method,
     target_pile_size: int = pr.target_pile_size,
     max_cell_size: Optional[float] = pr.max_cell_size,
@@ -885,6 +902,8 @@ def compute_divide_and_conquer_metacells(
     candidates_min_split_size_factor: Optional[float] = pr.candidates_min_split_size_factor,
     candidates_max_merge_size_factor: Optional[float] = pr.candidates_max_merge_size_factor,
     candidates_min_metacell_cells: Optional[int] = pr.min_metacell_cells,
+    candidates_max_split_min_cut_strength: Optional[float] = pr.max_split_min_cut_strength,
+    candidates_min_cut_seed_cells: Optional[int] = pr.min_cut_seed_cells,
     must_complete_cover: bool = False,
     final_max_outliers_levels: Optional[int] = pr.final_max_outliers_levels,
     deviants_min_gene_fold_factor: float = pr.deviants_min_gene_fold_factor,
@@ -1041,6 +1060,8 @@ def compute_divide_and_conquer_metacells(
                                      feature_min_gene_total=feature_min_gene_total,
                                      feature_min_gene_top3=feature_min_gene_top3,
                                      feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                     feature_gene_names=feature_gene_names,
+                                     feature_gene_patterns=feature_gene_patterns,
                                      forbidden_gene_names=forbidden_gene_names,
                                      forbidden_gene_patterns=forbidden_gene_patterns,
                                      cells_similarity_value_normalization=cells_similarity_value_normalization,
@@ -1063,6 +1084,8 @@ def compute_divide_and_conquer_metacells(
                                      candidates_min_split_size_factor=candidates_min_split_size_factor,
                                      candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                      candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                     candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                     candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                      must_complete_cover=must_complete_cover,
                                      deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
                                      deviants_max_gene_fraction=deviants_max_gene_fraction,
@@ -1086,14 +1109,17 @@ def compute_divide_and_conquer_metacells(
                                     feature_downsample_min_samples=feature_downsample_min_samples,
                                     feature_downsample_min_cell_quantile=feature_downsample_min_cell_quantile,
                                     feature_downsample_max_cell_quantile=feature_downsample_max_cell_quantile,
-                                    feature_min_gene_relative_variance=feature_min_gene_relative_variance,
                                     feature_min_gene_total=feature_min_gene_total,
                                     feature_min_gene_top3=feature_min_gene_top3,
+                                    feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                    feature_gene_names=feature_gene_names,
+                                    feature_gene_patterns=feature_gene_patterns,
                                     forbidden_gene_names=forbidden_gene_names,
                                     forbidden_gene_patterns=forbidden_gene_patterns,
                                     cells_similarity_value_normalization=cells_similarity_value_normalization,
                                     cells_similarity_log_data=cells_similarity_log_data,
                                     cells_similarity_method=cells_similarity_method,
+                                    groups_similarity_log_data=groups_similarity_log_data,
                                     groups_similarity_method=groups_similarity_method,
                                     target_pile_size=target_pile_size,
                                     pile_min_split_size_factor=pile_min_split_size_factor,
@@ -1116,6 +1142,8 @@ def compute_divide_and_conquer_metacells(
                                     candidates_min_split_size_factor=candidates_min_split_size_factor,
                                     candidates_max_merge_size_factor=None,
                                     candidates_min_metacell_cells=None,
+                                    candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                    candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                     must_complete_cover=True,
                                     final_max_outliers_levels=None,
                                     deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
@@ -1140,11 +1168,14 @@ def compute_divide_and_conquer_metacells(
                                                  feature_min_gene_total=feature_min_gene_total,
                                                  feature_min_gene_top3=feature_min_gene_top3,
                                                  feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                                 feature_gene_names=feature_gene_names,
+                                                 feature_gene_patterns=feature_gene_patterns,
                                                  forbidden_gene_names=forbidden_gene_names,
                                                  forbidden_gene_patterns=forbidden_gene_patterns,
                                                  cells_similarity_value_normalization=cells_similarity_value_normalization,
-                                                 cells_similarity_log_data=cells_similarity_log_data,
+                                                 cells_similarity_log_data=groups_similarity_log_data,
                                                  cells_similarity_method=groups_similarity_method,
+                                                 groups_similarity_log_data=groups_similarity_log_data,
                                                  groups_similarity_method=groups_similarity_method,
                                                  target_pile_size=target_pile_size,
                                                  pile_min_split_size_factor=pile_min_split_size_factor,
@@ -1167,6 +1198,8 @@ def compute_divide_and_conquer_metacells(
                                                  candidates_min_split_size_factor=pile_min_split_size_factor,
                                                  candidates_max_merge_size_factor=None,
                                                  candidates_min_metacell_cells=None,
+                                                 candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                                 candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                                  must_complete_cover=True,
                                                  final_max_outliers_levels=None,
                                                  deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
@@ -1196,14 +1229,17 @@ def compute_divide_and_conquer_metacells(
                                     feature_downsample_min_samples=feature_downsample_min_samples,
                                     feature_downsample_min_cell_quantile=feature_downsample_min_cell_quantile,
                                     feature_downsample_max_cell_quantile=feature_downsample_max_cell_quantile,
-                                    feature_min_gene_relative_variance=feature_min_gene_relative_variance,
                                     feature_min_gene_total=feature_min_gene_total,
                                     feature_min_gene_top3=feature_min_gene_top3,
+                                    feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                    feature_gene_names=feature_gene_names,
+                                    feature_gene_patterns=feature_gene_patterns,
                                     forbidden_gene_names=forbidden_gene_names,
                                     forbidden_gene_patterns=forbidden_gene_patterns,
                                     cells_similarity_value_normalization=cells_similarity_value_normalization,
                                     cells_similarity_log_data=cells_similarity_log_data,
                                     cells_similarity_method=cells_similarity_method,
+                                    groups_similarity_log_data=groups_similarity_log_data,
                                     groups_similarity_method=groups_similarity_method,
                                     target_pile_size=target_pile_size,
                                     pile_min_split_size_factor=pile_min_split_size_factor,
@@ -1226,6 +1262,8 @@ def compute_divide_and_conquer_metacells(
                                     candidates_min_split_size_factor=candidates_min_split_size_factor,
                                     candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                     candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                    candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                    candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                     must_complete_cover=must_complete_cover,
                                     final_max_outliers_levels=final_max_outliers_levels,
                                     deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
@@ -1253,14 +1291,17 @@ def compute_piled_metacells(
     feature_downsample_min_samples: int = pr.feature_downsample_min_samples,
     feature_downsample_min_cell_quantile: float = pr.feature_downsample_min_cell_quantile,
     feature_downsample_max_cell_quantile: float = pr.feature_downsample_max_cell_quantile,
-    feature_min_gene_total: int,
-    feature_min_gene_top3: int,
-    feature_min_gene_relative_variance: float,
+    feature_min_gene_total: Optional[int],
+    feature_min_gene_top3: Optional[int],
+    feature_min_gene_relative_variance: Optional[float],
+    feature_gene_names: Optional[Collection[str]] = None,
+    feature_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     forbidden_gene_names: Optional[Collection[str]],
     forbidden_gene_patterns: Optional[Collection[Union[str, Pattern]]],
     cells_similarity_value_normalization: float,
     cells_similarity_log_data: bool,
     cells_similarity_method: str,
+    groups_similarity_log_data: bool,
     groups_similarity_method: str,
     target_pile_size: int,
     pile_min_split_size_factor: float,
@@ -1283,6 +1324,8 @@ def compute_piled_metacells(
     candidates_min_split_size_factor: Optional[float],
     candidates_max_merge_size_factor: Optional[float],
     candidates_min_metacell_cells: Optional[int] = pr.min_metacell_cells,
+    candidates_max_split_min_cut_strength: Optional[float] = pr.max_split_min_cut_strength,
+    candidates_min_cut_seed_cells: Optional[int] = pr.min_cut_seed_cells,
     must_complete_cover: bool,
     final_max_outliers_levels: Optional[int],
     deviants_min_gene_fold_factor: float,
@@ -1312,6 +1355,8 @@ def compute_piled_metacells(
                             feature_min_gene_total=feature_min_gene_total,
                             feature_min_gene_top3=feature_min_gene_top3,
                             feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                            feature_gene_names=feature_gene_names,
+                            feature_gene_patterns=feature_gene_patterns,
                             forbidden_gene_names=forbidden_gene_names,
                             forbidden_gene_patterns=forbidden_gene_patterns,
                             cells_similarity_value_normalization=cells_similarity_value_normalization,
@@ -1334,6 +1379,8 @@ def compute_piled_metacells(
                             candidates_min_split_size_factor=candidates_min_split_size_factor,
                             candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                             candidates_min_metacell_cells=candidates_min_metacell_cells,
+                            candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                            candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                             must_complete_cover=must_complete_cover and piles_count == 1,
                             deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
                             deviants_max_gene_fraction=deviants_max_gene_fraction,
@@ -1392,11 +1439,14 @@ def compute_piled_metacells(
                                                  feature_min_gene_total=feature_min_gene_total,
                                                  feature_min_gene_top3=feature_min_gene_top3,
                                                  feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                                 feature_gene_names=feature_gene_names,
+                                                 feature_gene_patterns=feature_gene_patterns,
                                                  forbidden_gene_names=forbidden_gene_names,
                                                  forbidden_gene_patterns=forbidden_gene_patterns,
                                                  cells_similarity_value_normalization=cells_similarity_value_normalization,
                                                  cells_similarity_log_data=cells_similarity_log_data,
                                                  cells_similarity_method=cells_similarity_method,
+                                                 groups_similarity_log_data=groups_similarity_log_data,
                                                  groups_similarity_method=groups_similarity_method,
                                                  target_pile_size=target_pile_size,
                                                  pile_min_split_size_factor=pile_min_split_size_factor,
@@ -1418,6 +1468,8 @@ def compute_piled_metacells(
                                                  candidates_min_split_size_factor=candidates_min_split_size_factor,
                                                  candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                                  candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                                 candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                                 candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                                  must_complete_cover=must_complete_cover,
                                                  final_max_outliers_levels=final_max_outliers_levels,
                                                  deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
@@ -1448,9 +1500,11 @@ def _run_parallel_piles(
     feature_downsample_min_samples: int = pr.feature_downsample_min_samples,
     feature_downsample_min_cell_quantile: float = pr.feature_downsample_min_cell_quantile,
     feature_downsample_max_cell_quantile: float = pr.feature_downsample_max_cell_quantile,
-    feature_min_gene_total: int,
-    feature_min_gene_top3: int,
-    feature_min_gene_relative_variance: float,
+    feature_min_gene_total: Optional[int],
+    feature_min_gene_top3: Optional[int],
+    feature_min_gene_relative_variance: Optional[float],
+    feature_gene_names: Optional[Collection[str]] = None,
+    feature_gene_patterns: Optional[Collection[Union[str, Pattern]]] = None,
     forbidden_gene_names: Optional[Collection[str]],
     forbidden_gene_patterns: Optional[Collection[Union[str, Pattern]]],
     cells_similarity_value_normalization: float,
@@ -1473,6 +1527,8 @@ def _run_parallel_piles(
     candidates_min_split_size_factor: Optional[float],
     candidates_max_merge_size_factor: Optional[float],
     candidates_min_metacell_cells: Optional[int],
+    candidates_max_split_min_cut_strength: Optional[float] = pr.max_split_min_cut_strength,
+    candidates_min_cut_seed_cells: Optional[int] = pr.min_cut_seed_cells,
     must_complete_cover: bool,
     deviants_min_gene_fold_factor: float,
     deviants_max_gene_fraction: Optional[float],
@@ -1497,6 +1553,8 @@ def _run_parallel_piles(
                                  feature_min_gene_total=feature_min_gene_total,
                                  feature_min_gene_top3=feature_min_gene_top3,
                                  feature_min_gene_relative_variance=feature_min_gene_relative_variance,
+                                 feature_gene_names=feature_gene_names,
+                                 feature_gene_patterns=feature_gene_patterns,
                                  forbidden_gene_names=forbidden_gene_names,
                                  forbidden_gene_patterns=forbidden_gene_patterns,
                                  cells_similarity_value_normalization=cells_similarity_value_normalization,
@@ -1519,6 +1577,8 @@ def _run_parallel_piles(
                                  candidates_min_split_size_factor=candidates_min_split_size_factor,
                                  candidates_max_merge_size_factor=candidates_max_merge_size_factor,
                                  candidates_min_metacell_cells=candidates_min_metacell_cells,
+                                 candidates_max_split_min_cut_strength=candidates_max_split_min_cut_strength,
+                                 candidates_min_cut_seed_cells=candidates_min_cut_seed_cells,
                                  must_complete_cover=must_complete_cover,
                                  deviants_min_gene_fold_factor=deviants_min_gene_fold_factor,
                                  deviants_max_gene_fraction=deviants_max_gene_fraction,
