@@ -19,12 +19,14 @@ analysis pipeline.
 
 import re
 import sys
-from math import ceil, floor
+from math import ceil, floor, sqrt
 from re import Pattern
 from typing import (Any, Callable, Collection, List, Optional, Tuple, TypeVar,
                     Union, overload)
 from warnings import catch_warnings, filterwarnings, warn
 
+import cvxpy
+import igraph as ig  # type: ignore
 import numpy as np
 import pandas as pd  # type: ignore
 import scipy.sparse as sp  # type: ignore
@@ -42,7 +44,11 @@ __all__ = [
     'sort_compressed_indices',
 
     'corrcoef',
+    'cross_corrcoef_rows',
+    'pairs_corrcoef_rows',
     'logistics',
+    'cross_logistics_rows',
+    'pairs_logistics_rows',
 
     'log_data',
 
@@ -99,6 +105,9 @@ __all__ = [
 
     'random_piles',
     'group_piles',
+
+    'represent',
+    'min_cut',
 ]
 
 
@@ -380,7 +389,8 @@ def corrcoef(
 ) -> utt.NumpyMatrix:
     '''
     Similar to for ``numpy.corrcoef``, but also works for a sparse ``matrix``, and can be
-    ``reproducible`` regardless of the number of cores used (at the cost of some slowdown).
+    ``reproducible`` regardless of the number of cores used (at the cost of some slowdown). It only
+    works for matrices with a float or double element data type.
 
     If ``reproducible``, a slower (still parallel) but reproducible algorithm will be used.
 
@@ -398,24 +408,19 @@ def corrcoef(
 
         The result is always dense, as even for sparse data, the correlation is rarely exactly zero.
     '''
-    if not reproducible:
-        return _corrcoef_fast(matrix, per)
+    per, dense = _get_dense_for('corrcoef', matrix, per)
 
-    _, dense, compressed = utt.to_proper_matrices(matrix)
+    if not reproducible or str(dense.dtype) not in ('float', 'double', 'float32', 'float64'):
+        return _corrcoef_fast(dense, per)
 
-    if compressed is not None:
-        return _corrcoef_compressed_matrix(compressed, per)
-
-    assert dense is not None
-    return _corrcoef_dense_matrix(dense, per)
+    return _corrcoef_reproducible(dense, per)
 
 
 @utm.timed_call('.irreproducible')
 def _corrcoef_fast(
-    matrix: utt.Matrix,
-    per: Optional[str],
+    dense: utt.NumpyMatrix,
+    per: str,
 ) -> utt.NumpyMatrix:
-    per, dense = _get_dense_for('corrcoef', matrix, per)
     axis = utt.PER_OF_AXIS.index(per)
 
     utm.timed_parameters(results=dense.shape[axis],
@@ -440,12 +445,11 @@ def _corrcoef_fast(
     return result
 
 
-@utm.timed_call('.dense')
-def _corrcoef_dense_matrix(
+@utm.timed_call('.reproducible')
+def _corrcoef_reproducible(
     dense: utt.NumpyMatrix,
-    per: Optional[str],
+    per: str,
 ) -> utt.NumpyMatrix:
-    per, dense = _get_dense_for('corrcoef', dense, per)
     if per == 'column':
         dense = dense.T
     utm.timed_parameters(results=dense.shape[0],
@@ -457,43 +461,108 @@ def _corrcoef_dense_matrix(
     return result
 
 
-@utm.timed_call('.compressed')
-def _corrcoef_compressed_matrix(
-    compressed: utt.CompressedMatrix,
-    per: Optional[str]
+@utm.timed_call()
+def cross_corrcoef_rows(
+    first_matrix: utt.NumpyMatrix,
+    second_matrix: utt.NumpyMatrix,
+    *,
+    reproducible: bool,  # pylint: disable=unused-argument
 ) -> utt.NumpyMatrix:
-    if per is not None:
-        assert utt.is_layout(compressed, f'{per}_major')
-        if per == 'column':
-            compressed = compressed.transpose()
+    '''
+    Similar to for ``numpy.corrcoef``, but computes the correlations between each row of the
+    ``first_matrix`` and each row of the ``second_matrix``. The result matrix contains one row per
+    row of the first matrix and one column per row of the second matrix. Both matrices must be
+    dense, in row-major layout, have the same (float or double) element data type, and contain the
+    same number of columns.
 
-    utm.timed_parameters(results=compressed.shape[0],
-                         elements=compressed.shape[1],
-                         nnz=compressed.nnz)
-    extension_name = 'correlate_compressed_%s_t_%s_t_%s_t' \
-        % (compressed.data.dtype, compressed.indices.dtype, compressed.indptr.dtype)
-    extension = getattr(xt, extension_name)
-    result = np.empty((compressed.shape[0], compressed.shape[0]),
+    If ``reproducible``, a slower (still parallel) but reproducible algorithm will be used.
+
+    Unlike ``numpy.corrcoef``, if given a row with identical values, instead of complaining about
+    division by zero, this will report a zero correlation. This makes sense for the intended usage
+    of computing similarities between cells/genes - an all-zero row has no data so we declare it to
+    be "not similar" to anything else.
+
+    .. note::
+
+        This only works for floating-point matrices.
+
+    .. todo::
+
+        Implement a fast algorithm for the non-reproducible case.
+    '''
+    first_matrix = utt.mustbe_numpy_matrix(first_matrix)
+    second_matrix = utt.mustbe_numpy_matrix(second_matrix)
+    assert utt.matrix_layout(first_matrix) == 'row_major'
+    assert utt.matrix_layout(second_matrix) == 'row_major'
+    assert first_matrix.shape[1] == second_matrix.shape[1]
+    assert first_matrix.dtype == second_matrix.dtype
+
+    extension_name = 'cross_correlate_dense_%s_t' % first_matrix.dtype
+    result = np.empty((first_matrix.shape[0], second_matrix.shape[0]),
                       dtype='float32')
     extension = getattr(xt, extension_name)
-    extension(compressed.data, compressed.indices,
-              compressed.indptr, compressed.shape[1], result)
+    extension(first_matrix, second_matrix, result)
+    return result
+
+
+@utm.timed_call()
+def pairs_corrcoef_rows(
+    first_matrix: utt.NumpyMatrix,
+    second_matrix: utt.NumpyMatrix,
+    *,
+    reproducible: bool,  # pylint: disable=unused-argument
+) -> utt.NumpyMatrix:
+    '''
+    Similar to for ``numpy.corrcoef``, but computes the correlations between each row of the
+    ``first_matrix`` and each matching row of the ``second_matrix``. Both matrices must be dense, in
+    row-major layout, have the same (float or double) element data type, and the same shape.
+
+    If ``reproducible``, a slower (still parallel) but reproducible algorithm will be used.
+
+    Unlike ``numpy.corrcoef``, if given a row with identical values, instead of complaining about
+    division by zero, this will report a zero correlation. This makes sense for the intended usage
+    of computing similarities between cells/genes - an all-zero row has no data so we declare it to
+    be "not similar" to anything else.
+
+    .. note::
+
+        This only works for floating-point matrices.
+
+    .. todo::
+
+        Implement a fast algorithm for the non-reproducible case.
+    '''
+    first_matrix = utt.mustbe_numpy_matrix(first_matrix)
+    second_matrix = utt.mustbe_numpy_matrix(second_matrix)
+    assert utt.matrix_layout(first_matrix) == 'row_major'
+    assert utt.matrix_layout(second_matrix) == 'row_major'
+    assert first_matrix.shape == second_matrix.shape
+    assert first_matrix.dtype == second_matrix.dtype
+
+    extension_name = 'pairs_correlate_dense_%s_t' % first_matrix.dtype
+    result = np.empty(first_matrix.shape[0], dtype='float32')
+    extension = getattr(xt, extension_name)
+    extension(first_matrix, second_matrix, result)
     return result
 
 
 @utm.timed_call()
 def logistics(
-    matrix: utt.Matrix,
+    matrix: utt.NumpyMatrix,
     *,
-    location: float = 0.8,
-    scale: float = 5,
+    location: float,
+    slope: float,
     per: Optional[str]
 ) -> utt.NumpyMatrix:
     '''
-    Compute a similarity matrix, similar to for ``numpy.corrcoef``, but uses the logistics function.
+    Compute a matrix of distances between each pair of rows in a dense (float or double) matrix
+    using the logistics function.
 
-    This computes, for each pair of vectors, the mean of ``1/(1+exp(-scale*(abs(x-y)-location)))``
-    of each of the elements. Typically this is applied to the log of the raw data.
+    The raw value of the logistics distance between a pair of vectors ``x`` and ``y`` is the mean of
+    ``1/(1+exp(-slope*(abs(x[i]-y[i])-location)))``. This has a minimum of
+    ``1/(1+exp(slope*location))`` for identical vectors and an (asymptotic) maximum of 1. We
+    normalize this to a range between 0 and 1, to be useful as a distance measure (with a zero
+    distance between identical vectors).
 
     If ``per`` is ``None``, the matrix must be square and is assumed to be symmetric, so the most
     efficient direction is used based on the matrix layout. Otherwise it must be one of ``row`` or
@@ -517,10 +586,67 @@ def logistics(
     if per == 'column':
         dense = dense.T
 
-    extension_name = 'logistics_dense_matrix_%s_t' % dense.dtype
+    extension_name = 'logistics_dense_%s_t' % dense.dtype
     result = np.empty((dense.shape[0], dense.shape[0]), dtype='float32')
     extension = getattr(xt, extension_name)
-    extension(dense, result, location, scale)
+    extension(dense, result, location, slope)
+    return result
+
+
+@utm.timed_call()
+def cross_logistics_rows(
+    first_matrix: utt.NumpyMatrix,
+    second_matrix: utt.NumpyMatrix,
+    *,
+    location: float,
+    slope: float,
+) -> utt.NumpyMatrix:
+    '''
+    Similar to for :py:func:`logistics`, but computes the distances between each row of the
+    ``first_matrix`` and each row of the ``second_matrix``. The result matrix contains one row per
+    row of the first matrix and one column per row of the second matrix. Both matrices must be
+    dense, in row-major layout, have the same (float or double) element data type, and contain the
+    same number of columns.
+    '''
+    first_matrix = utt.mustbe_numpy_matrix(first_matrix)
+    second_matrix = utt.mustbe_numpy_matrix(second_matrix)
+    assert utt.matrix_layout(first_matrix) == 'row_major'
+    assert utt.matrix_layout(second_matrix) == 'row_major'
+    assert first_matrix.shape[1] == second_matrix.shape[1]
+    assert first_matrix.dtype == second_matrix.dtype
+
+    extension_name = 'cross_logistics_dense_%s_t' % first_matrix.dtype
+    result = np.empty((first_matrix.shape[0], second_matrix.shape[0]),
+                      dtype='float32')
+    extension = getattr(xt, extension_name)
+    extension(first_matrix, second_matrix, location, slope, result)
+    return result
+
+
+@utm.timed_call()
+def pairs_logistics_rows(
+    first_matrix: utt.NumpyMatrix,
+    second_matrix: utt.NumpyMatrix,
+    *,
+    location: float,
+    slope: float,
+) -> utt.NumpyVector:
+    '''
+    Similar to for :py:func:`logistics`, but computes the distances between each row of the
+    ``first_matrix`` and each matching row of the ``second_matrix``. Both matrices must be dense, in
+    row-major layout, have the same (float or double) element data type, and the same shape.
+    '''
+    first_matrix = utt.mustbe_numpy_matrix(first_matrix)
+    second_matrix = utt.mustbe_numpy_matrix(second_matrix)
+    assert utt.matrix_layout(first_matrix) == 'row_major'
+    assert utt.matrix_layout(second_matrix) == 'row_major'
+    assert first_matrix.shape == second_matrix.shape
+    assert first_matrix.dtype == second_matrix.dtype
+
+    extension_name = 'pairs_logistics_dense_%s_t' % first_matrix.dtype
+    result = np.empty(first_matrix.shape[0], dtype='float32')
+    extension = getattr(xt, extension_name)
+    extension(first_matrix, second_matrix, location, slope, result)
     return result
 
 
@@ -694,7 +820,7 @@ def _downsample_dense_matrix(
             extension(input_array, output_array, samples, random_seed)
     else:
         extension_name = \
-            'downsample_matrix_%s_t_%s_t' % (matrix.dtype, output.dtype)
+            'downsample_dense_%s_t_%s_t' % (matrix.dtype, output.dtype)
         extension = getattr(xt, extension_name)
         with utm.timed_step('extensions.downsample_dense_matrix'):
             utm.timed_parameters(results=results_count,
@@ -1129,11 +1255,13 @@ def top_per(matrix: utt.Matrix, top: int, *, per: Optional[str], ranks: bool = F
 
     else:
         indices = np.empty(top * size, dtype='int32')
-        data = np.empty(top * size, dtype='float32')
+        data = np.empty(top * size, dtype=dense.dtype)
 
         with utm.timed_step('extensions.collect_top'):
-            utm.timed_parameters(size=size, keep=top)
-            xt.collect_top(top, dense, indices, data, ranks)
+            utm.timed_parameters(size=size, keep=top, dtype=str(dense.dtype))
+            extension_name = 'collect_top_%s_t' % dense.dtype
+            extension = getattr(xt, extension_name)
+            extension(top, dense, indices, data, ranks)
 
     top_data = sp.csr_matrix((data, indices, indptr), shape=dense.shape)
     top_data.has_sorted_indices = True
@@ -1158,20 +1286,24 @@ def prune_per(compressed: utt.CompressedMatrix, top: int) -> utt.CompressedMatri
         assert layout == 'column_major'
         size = compressed.shape[1]
 
-    indptr_array = np.empty(1 + size, dtype='int64')
-    indices_array = np.empty(size * top, dtype='int32')
-    data_array = np.empty(size * top, dtype='float32')
-    assert utt.matrix_dtype(compressed) == 'float32'
+    data_array = np.empty(size * top, dtype=compressed.data.dtype)
+    indices_array = np.empty(size * top, dtype=compressed.indices.dtype)
+    indptr_array = np.empty(1 + size, dtype=compressed.indptr.dtype)
 
     with utm.timed_step('extensions.collect_pruned'):
         utm.timed_parameters(size=size, nnz=compressed.nnz, keep=top)
-        xt.collect_pruned(top,
-                          compressed.data,
-                          compressed.indices,
-                          compressed.indptr,
-                          data_array,
-                          indices_array,
-                          indptr_array)
+        extension_name = 'collect_pruned_%s_t_%s_t_%s_t' \
+            % (compressed.data.dtype,
+               compressed.indices.dtype,
+               compressed.indptr.dtype)
+        extension = getattr(xt, extension_name)
+        extension(top,
+                  compressed.data,
+                  compressed.indices,
+                  compressed.indptr,
+                  data_array,
+                  indices_array,
+                  indptr_array)
 
     if layout == 'row_major':
         constructor = sp.csr_matrix
@@ -1660,8 +1792,8 @@ def patterns_matches(
     invert: bool = False,
 ) -> utt.NumpyVector:
     '''
-    Given a collection of ``strings``, return a boolean mask specifying which of them match the
-    given regular expression ``patterns``.
+    Given a collection of (case-insensitive) ``strings``, return a boolean mask specifying which of
+    them match the given regular expression ``patterns``.
 
     If ``invert`` (default: {invert}), invert the mask.
     '''
@@ -1670,10 +1802,13 @@ def patterns_matches(
 
     utm.timed_parameters(patterns=len(patterns), strings=len(strings))
 
-    pattern: Pattern = \
-        re.compile('|'.join([alternative if isinstance(alternative, str)
-                             else alternative.pattern
-                             for alternative in patterns]))
+    unified_pattern = \
+        '(' \
+        + ')|('.join([alternative if isinstance(alternative, str)
+                      else alternative.pattern
+                      for alternative in patterns]) \
+        + ')'
+    pattern: Pattern = re.compile(unified_pattern, re.IGNORECASE)
 
     mask = np.array([bool(pattern.match(string)) for string in strings])
 
@@ -1954,7 +2089,7 @@ def shuffle_matrix(
         assert dense is not None
         if per == 'column':
             dense = dense.transpose()
-        extension_name = 'shuffle_matrix_%s_t' % dense.dtype
+        extension_name = 'shuffle_dense_%s_t' % dense.dtype
         extension = getattr(xt, extension_name)
         extension(dense, random_seed)
 
@@ -2084,3 +2219,118 @@ def group_piles(
     assert np.min(group_of_groups) == 0
     group_of_group_of_elements = group_of_groups[group_of_elements]
     return group_of_group_of_elements
+
+
+@utm.timed_call()
+def represent(
+    goal: utt.NumpyVector,
+    basis: utt.NumpyMatrix,
+) -> Optional[Tuple[float, utt.NumpyVector]]:
+    '''
+    Represent a ``goal`` vector as a weighted average of the row vectors of some ``basis`` matrix.
+
+    This computes a non-negative weight for each matrix row, such that the sum of weights is 1,
+    minimizing the distance (L2 norm) between the goal vector and the weighted average of the basis
+    vectors. This is a convex problem quadratic subject to a linear constraint, so ``cvxpy`` solves
+    it efficiently.
+
+    The return value is a tuple with the score of the weights vector, and the weights vector itself.
+    '''
+    rows_count, columns_count = basis.shape
+    assert columns_count == len(goal)
+
+    variables = cvxpy.Variable(rows_count, nonneg=True)
+    constraints = [cvxpy.sum(variables) == 1]
+    objective = cvxpy.norm(goal - variables @ basis, 2)
+
+    problem = cvxpy.Problem(cvxpy.Minimize(objective), constraints)
+    result = problem.solve()
+
+    if result is None:
+        return None
+    return (float(result), utt.to_numpy_vector(variables.value))
+
+
+@utm.timed_call()
+def min_cut(  # pylint: disable=too-many-branches,too-many-statements
+    weights: utt.Matrix
+) -> Tuple[ig.Cut, Optional[float]]:
+    '''
+    Find the minimal cut that will split an undirected graph (with a symmetrical ``weights``
+    matrix).
+
+    Returns the ``igraph.Cut`` object describing the cut, and the scale-invariant strength of the
+    cut edges. This strength is the ratio between the mean weight of an edge connecting a random
+    node in each partition and the mean weight of an edge connecting two random nodes inside a
+    random partition. If either of the partitions contains no edges (e.g. contains a single node),
+    the strength will be ``None``.
+    '''
+    assert weights.shape[0] == weights.shape[1]
+    nodes_count = weights.shape[0]
+    edges: List[Tuple[int, int]] = []
+    weight_of_edges: List[float] = []
+
+    _, dense, compressed = utt.to_proper_matrices(weights)
+    if dense is not None:
+        for source in range(nodes_count):
+            for target in range(source):
+                weight = dense[source, target]
+                if weight > 0:
+                    edges.append((source, target))
+                    weight_of_edges.append(weight)
+    else:
+        assert compressed is not None
+        for source in range(nodes_count):
+            offsets = range(compressed.indptr[source],
+                            compressed.indptr[source + 1])
+            for target, weight in zip(compressed.indices[offsets], compressed.data[offsets]):
+                if target >= source:
+                    break
+                edges.append((source, target))
+                weight_of_edges.append(weight)
+
+    graph = ig.Graph(n=nodes_count, edges=edges)
+    cut = graph.mincut(capacity=weight_of_edges)
+
+    is_second = np.zeros(nodes_count, dtype='bool')
+    is_second[cut.partition[1]] = True
+    first_size = len(cut.partition[0])
+    second_size = len(cut.partition[1])
+    assert first_size + second_size == nodes_count
+
+    cut_total_weight = 0.0
+    first_total_weight = 0.0
+    second_total_weight = 0.0
+    for ((source, target), weight) in zip(edges, weight_of_edges):
+        if is_second[source]:
+            if is_second[target]:
+                second_total_weight += weight
+            else:
+                cut_total_weight += weight
+        else:
+            if is_second[target]:
+                cut_total_weight += weight
+            else:
+                first_total_weight += weight
+
+    if cut_total_weight == 0:
+        return (cut, 0.0)
+
+    if first_total_weight > 0 and second_total_weight > 0:
+        cut_edges_count = first_size * second_size
+        first_edges_count = (first_size * (first_size - 1)) / 2
+        second_edges_count = (second_size * (second_size - 1)) / 2
+        total_edges_count = (nodes_count * (nodes_count - 1)) / 2
+        assert cut_edges_count + first_edges_count + \
+            second_edges_count == total_edges_count
+
+        cut_mean_weight = cut_total_weight / cut_edges_count
+        first_mean_weight = first_total_weight / first_edges_count
+        second_mean_weight = second_total_weight / second_edges_count
+        inner_mean_weight = sqrt(first_mean_weight * second_mean_weight)
+
+        cut_strength: Optional[float] = cut_mean_weight / inner_mean_weight
+    else:
+        cut_strength = None
+
+    return (cut, cut_strength)
